Package: reshape2
Title: Flexibly Reshape Data: A Reboot of the Reshape Package
Version: 1.4.5
Authors@R: 
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre"))
Description: Flexibly restructure and aggregate data using just two
    functions: melt and 'dcast' (or 'acast').
License: MIT + file LICENSE
URL: https://github.com/hadley/reshape
BugReports: https://github.com/hadley/reshape/issues
Depends: R (>= 3.1)
Imports: plyr (>= 1.8.1), Rcpp, stringr
Suggests: covr, lattice, testthat (>= 3.0.0)
LinkingTo: Rcpp
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2025-11-11 14:12:59 UTC; hadleywickham
Author: Hadley Wickham [aut, cre]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2025-11-12 06:20:11 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 21:14:03 UTC; unix
Archs: reshape2.so.dSYM
