# Generated by roxygen2: do not edit by hand

S3method(melt,array)
S3method(melt,data.frame)
S3method(melt,default)
S3method(melt,list)
S3method(melt,matrix)
S3method(melt,table)
export(acast)
export(add_margins)
export(colsplit)
export(dcast)
export(melt)
export(recast)
import(stringr)
importFrom(Rcpp,evalCpp)
importFrom(plyr,alply)
importFrom(plyr,amv_dimnames)
importFrom(plyr,as.quoted)
importFrom(plyr,eval.quoted)
importFrom(plyr,id)
importFrom(plyr,llply)
importFrom(plyr,rbind.fill)
importFrom(plyr,split_labels)
importFrom(plyr,vaggregate)
importFrom(stats,setNames)
importFrom(utils,type.convert)
useDynLib(reshape2)
